//
//  saomiaoViewController.m
//  原生扫描生成demo
//
//  Created by mac on 16/11/16.
//  Copyright © 2016年 TXZhongJiaowang. All rights reserved.
//

#import "saomiaoViewController.h"
#import <AVFoundation/AVFoundation.h>
#import "layoutView.h"

#define kZero 0
#define kFullScreen [UIScreen mainScreen].bounds
#define kFullWidth [UIScreen mainScreen].bounds.size.width
#define kFullHeight [UIScreen mainScreen].bounds.size.height

@interface saomiaoViewController ()<AVCaptureMetadataOutputObjectsDelegate>
@property (nonatomic,strong)AVCaptureSession *session;
@end

@implementation saomiaoViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view
    self.automaticallyAdjustsScrollViewInsets = NO;
    //检查相机权限
    [self checkAVAuthorizationStatus];
    
    layoutView *layout = [[layoutView alloc]initWithFrame:CGRectMake(kZero, 64, kFullWidth, kFullHeight-64)];
    [self.view addSubview:layout];
    
    AVCaptureVideoPreviewLayer *layer = [AVCaptureVideoPreviewLayer layerWithSession:self.session];
    layer.videoGravity = AVLayerVideoGravityResizeAspectFill;
    layer.frame = self.view.layer.bounds;
    [self.view.layer insertSublayer:layer atIndex:0];
}

/*
 *  AVAuthorizationStatus
 *
 *  @param  AVAuthorizationStatusRestricted         授权请求
 *  @param  AVAuthorizationStatusDenied             授权失败
 *  @param  AVAuthorizationStatusAuthorized         授权成功
 */

- (void)checkAVAuthorizationStatus{
    AVAuthorizationStatus status = [AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeVideo];
    if (status == AVAuthorizationStatusRestricted) {
        
    }else if (status == AVAuthorizationStatusDenied){
        UIAlertController *alertC = [UIAlertController alertControllerWithTitle:@"请在设置中开启照相机权限" message:nil preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleDestructive handler:^(UIAlertAction * _Nonnull action) {
            
        }];
        [alertC addAction:action];
        [self presentViewController:alertC animated:YES completion:nil];
    }else if (status == AVAuthorizationStatusAuthorized){
        
    }
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [self.session startRunning];
    self.tabBarController.tabBar.hidden = YES;
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    [self.session stopRunning];
}

- (AVCaptureSession *)session{
    if (!_session) {
        //获取摄像设备
        AVCaptureDevice *device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
        //创建输入流
        AVCaptureDeviceInput *input = [AVCaptureDeviceInput deviceInputWithDevice:device error:nil];
        if (!input) {
            return nil;
        }
        
        //创建输出流
        AVCaptureMetadataOutput *output = [[AVCaptureMetadataOutput alloc]init];
        //设置代理 在主线程里刷新
        [output setMetadataObjectsDelegate:self queue:dispatch_get_main_queue()];
        //设置扫描区域的比例
        CGFloat width = 300/CGRectGetWidth(self.view.bounds);
        CGFloat height = 300/CGRectGetHeight(self.view.bounds);
#warning rectOfInterest --> (y,x,height,width)且默认(0,0,1,1)
        output.rectOfInterest = CGRectMake((1-height)/2, (1-width)/2, height, width);
        self.session = [[AVCaptureSession alloc]init];
        //高质量采集率
        [self.session setSessionPreset:AVCaptureSessionPresetHigh];
        [self.session addInput:input];
        [self.session addOutput:output];
        
        //设置扫码支持的编码格式(条形码和二维码兼容)
        output.metadataObjectTypes = @[AVMetadataObjectTypeQRCode,
                                       AVMetadataObjectTypeEAN13Code,
                                       AVMetadataObjectTypeEAN8Code,
                                       AVMetadataObjectTypeCode128Code];
        
        
    }
    return _session;
}
#pragma mark **** AVCaptureMetadataOutputObjectsDelegate
- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputMetadataObjects:(NSArray *)metadataObjects fromConnection:(AVCaptureConnection *)connection{
    NSLog(@"%@",metadataObjects);
    if (metadataObjects.count > 0) {
        [self.session stopRunning];
        
        AVMetadataMachineReadableCodeObject *metadataObject = [metadataObjects firstObject];
#pragma mark * 扫描结果事件
        UIAlertController *alertC = [UIAlertController alertControllerWithTitle:@"扫码结果" message:[metadataObject stringValue] preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:@"确定" style:UIAlertActionStyleDestructive handler:^(UIAlertAction * _Nonnull action) {
            [self.session startRunning];
        }];
        [alertC addAction:action];
        [self presentViewController:alertC animated:YES completion:nil];
    }
}
- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
